//////////////////////////////////////////////////////////////////////
//////               PIP-GRL 31415 wristcomputer                //////
////// Personal Information Processing - Graphical Routing Link //////
//////////////////////////////////////////////////////////////////////
// Not really sure of the acronym/abbreviation but it will      //////
// probably contain one or more of the following words:         //////
// Portable Personal Parallel Payload Pipeline Pixel Port       //////
// Interactive Interface Injection Interpreter Integrated       //////
// Graphical Gateway                                            //////
// Realtime Runtime Root Record                                 //////
// Link Login                                                   //////
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////
/////// INITIAL CONFIGURATION ///////
/////////////////////////////////////
// Define libraries
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_TFTLCD.h> // Hardware-specific library
#include <stdint.h>          // not sure why this is included
#include "TouchScreen.h"     // Touchscreen library
#include <Adafruit_GPS.h>
#include <SdFat.h>
#include <SdFatUtil.h>

// Configure pins for screen and speaker (more will follow)
#define YM 9   // can be a digital pin
#define XM A3  // must be an analog pin, use "An" notation!
#define YP A9  // must be an analog pin, use "An" notation!
#define XP 4   // can be a digital pin
#define LCD_CS A8 // Chip Select goes to Analog 3
#define LCD_CD A5 // Command/Data goes to Analog 2
#define LCD_WR A4 // LCD Write goes to Analog 1
#define LCD_RD A1 // LCD Read goes to Analog 0
#define LCD_RESET A2 // Can alternately just connect to Arduino's reset pin
#define SPEAKER 3 // Can alternately just connect to Arduino's reset pin
const uint8_t chipSelect = SS;
// GPS is set to serial1 (pin0-1, RX0/TX0)

// Configure microsd settings
SdFat sd; // set filesystem
SdFile file; // set file
#define BUF_SIZE 10240

// Configure screen settings
Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET); // setup TFT library

// Configure bmpDraw settings
#define BUFFPIXEL 20

// Configure touchsensitive settings
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300); // set touchscreen resistance to 300 ohm
Point p = ts.getPoint(); // define p as an x-y-z array
#define MINPRESSURE 10 // minimum pressure required to register a hit
#define MAXPRESSURE 1000 // maximum pressure required to register a hit (otherwise it has fallen)
#define TS_MINX 110 // define edges of screen v.s. edges of touchlayer
#define TS_MINY 120 // this is ofcourse in PORTRAIT MODE
#define TS_MAXX 930 // later on, a conversion to landscape mode is initiated
#define TS_MAXY 935 // remember this while testing

// Configure GPS settings
Adafruit_GPS GPS(&Serial1);
#define GPSECHO  false

// Assign human-readable names to some common 16-bit color values:
#define	BLACK        0x0000
#define	GREEN        0x07E0
#define RED          0xF800
#define BLUE    0x001F
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define PINK    0xFCDF
#define WHITE   0xFFFF

// set cout output to Serial
ArduinoOutStream cout(Serial);
// store error strings in flash to save RAM
#define error(s) sd.errorHalt_P(PSTR(s))

// define default settings for scripts
int hud_color = GREEN; // text color
int background_color = BLACK; // background color
int rotation = 3; // set screen to landscape
int touchx; // create touchx variable
int touchy; // create touchy variable
int touchz; // create touchz variable
boolean passwd = 0; // create passwd boolean with loginattempts counter.
int loginattempts = 0; // set loginattempts to 0, increments on incorrect attempts
boolean mute = 0; // set mute to false
int page = 0; // define page/menu/submenu and set these to 0 (login page)
int menu = 0;
int submenu = 0;
elapsedMillis millicounter_1000;
elapsedMillis millicounter_5000;
elapsedMillis millicounter_10000;
elapsedMillis millicounter_30000;
elapsedMillis millicounter_60000;
int currentminute = 0;
int currentsecond = 0;
int currenthour = 0;
int currentday = 0;
int currentmonth = 0;
int currentyear = 0;
float gps_lat = 0.0; // remember last known gps lat (float)
float gps_lon = 0.0; // remember last known gps lon (float)
char gps_lat2; // remember last known gps lat (char)
char gps_lon2; // remember last known gps lon (char)
float gps_lat_degrees = 0.0;
float gps_lon_degrees = 0.0;
int recalc_lat = 0; // recalculate gps_lat to screen relative location
int recalc_lon = 0; // recalculate gps_lon to screen relative location



// TEST and TEMP SECTION
long randNumber;
long average_randNumber = 0;
int average_counter = 0;
int batterylevel = 100;
boolean charging = 1;
////////////////////////////////////////////
/////// end of INITIAL CONFIGURATION ///////
////////////////////////////////////////////


/////////////////////
/////// SETUP ///////
/////////////////////
void setup(void) {
  Serial1.begin(115200); // connect at 115200 so we can read the GPS fast enough and echo without dropping chars
  // initialize GPS module
  GPS.begin(9600); // 9600 NMEA is the default baud rate for Adafruit MTK GPS's- some use 4800
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA); // uncomment this line to turn on RMC (recommended minimum) and GGA (fix data) including altitude
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);   // 1 Hz update rate
  delay(1000);

  // listen to analog pin A13 for random noise values
  randomSeed(analogRead(A13));

  // declare pin 3 as output for speaker:
  pinMode(SPEAKER, OUTPUT);

  // reset and initialize the TFT
  tft.reset();
  // fetch the TFT's ID
  uint16_t identifier = tft.readID();
  // initialize the TFT with proper ID
  tft.begin(identifier);
  // set TFT rotation/orientation to landscape
  tft.setRotation(rotation);

  // initialize the MicroSD adapter
  if (!sd.begin(chipSelect, SPI_FULL_SPEED)) sd.initErrorHalt();

  Page_0(); // goto login page
}
////////////////////////////
/////// end of SETUP ///////
////////////////////////////

